import subprocess
import logging
import threading
from datetime import datetime
from django.db import transaction
from django.utils import timezone
from typing import List, Dict, Any, Tuple, Optional
import re
import os

from django.conf import settings
from .models import Config, Server, SerialHistory, WiFiStatus, UpdateStatus
from .scheme import HostapdConfig


logger = logging.getLogger(__name__)


# NOTE нужно добавить обработку ошибки, когда закончился период серийного номера
"""(base) goodman@goodman-hp:/opt/kelvpn/bin$ sudo ./KelVPNCli -C 04
[sudo] пароль для goodman: 
Cmd::Connect: (QVariant(int, 4))
Connection state updated: Disconnected
Activating tasks...
Cmd::Connect: Task activated
Connection state updated: Connecting
Task::Connect: Connecting to server: Europe.Netherlands.5...
Error: Serial key expired
Task::Connect: Connection error signal caught! Abort.
Task::Connect: Connection failed signal caught! Abort.
Connection state updated: Disconnected
All tasks are finished.
(base) goodman@goodman-hp:/opt """


class KelVPNCliService:
    def __init__(self) -> None:
        self.cli_path: str = "/opt/kelvpn/bin/KelVPNCli"

    def execute_command(self, args: List[str], timeout: int = 90) -> str:
        """
        Executes a command with the provided arguments and returns the result.

        :param args: List of command arguments.
        :param timeout: Timeout for the command execution in seconds.
        :return: Standard output from the command.
        :raises Exception: If the command execution fails or times out.
        """
        try:
            result = subprocess.run(
                args, capture_output=True, text=True, check=True, timeout=timeout
            )
            return result.stdout
        except subprocess.CalledProcessError as e:
            logger.error(f"Command execution error: {e.stderr}")
            raise Exception(f"Command execution error: {e.stderr}") from e
        except subprocess.TimeoutExpired:
            logger.error("Command execution time exceeded the limit.")
            raise Exception("Command execution time exceeded the limit.") from None

    def start_service(self, server_id: str) -> Tuple[str, bool]:
        """
        Starts KelVPNCli with the selected server.
        
        :param server_id: The ID of the server to connect to.
        :return: Tuple containing the command output and a success flag.
        """
        cmd_connect: List[str] = [self.cli_path, "-C", server_id]
        
        try:
            output: str = self.execute_command(cmd_connect)
        except Exception as e:
            logger.error(f"Command execution error: {e}")
            return f"Command execution error: {e}", False

        # If there is a timeout error in the output, restart KelVPNService and try connecting again
        if "Error: The command execution time" in output:
            logger.error(f"Connection timed out. Output: {output}")
            self.update_config(is_connected=False)
            
            # Restarting KelVPNService
            cmd_restart_kelvpn: List[str] = [
                "systemctl",
                "restart",
                "KelVPNService.service",
            ]
            try:
                kelvpn_output: str = self.execute_command(cmd_restart_kelvpn)
                logger.info(f"KelVPNService restarted successfully: {kelvpn_output}")
            except Exception as e:
                logger.error(f"Failed to restart KelVPNService: {e}")
                return f"Failed to restart KelVPNService: {e}", False

            import time
            time.sleep(2)

            # Reconnecting attempt
            try:
                output_retry: str = self.execute_command(cmd_connect)
            except Exception as e:
                logger.error(f"Retry command execution error: {e}")
                return f"Retry command execution error: {e}", False

            if "State: Connect" in output_retry and "Success!" in output_retry:
                self.update_config(is_connected=True, is_activated=True)
                return output_retry, True
            else:
                self.update_config(is_connected=False)
                logger.error(f"Retry failed. Command output: {output_retry}")
                return output_retry, False

        if "State: Connect" not in output or "Success!" not in output:
            self.update_config(is_connected=False)
            logger.error(f"Failed to establish a connection. Command output: {output}")
            return output, False

        self.update_config(is_connected=True, is_activated=True)
        return output, True

    def disconnect_service(self) -> str:
        """
        Disconnects KelVPNCli and stops related services.
        
        :return: Combined output from the commands as a string.
        """
        outputs = []
        try:
            # Disconnect KelVPNCli
            cmd_disconnect: List[str] = [self.cli_path, "-D"]
            disconnect_output: str = self.execute_command(cmd_disconnect)
            outputs.append(f"KelVPNCli disconnected: {disconnect_output}")
            logger.info(outputs[-1])
        except Exception as e:
            outputs.append(f"KelVPNCli disconnect failed: {e}")
            logger.error(outputs[-1])
        
        # Stop KelVPNCheck.service
        try:
            if self.is_service_active("KelVPNCheck.service"):
                cmd_stop_check: List[str] = ["systemctl", "stop", "KelVPNCheck.service"]
                check_service_output: str = self.execute_command(cmd_stop_check)
                outputs.append(f"KelVPNCheck.service stopped: {check_service_output}")
                logger.info(outputs[-1])
            else:
                outputs.append("KelVPNCheck.service is not running or not found.")
                logger.warning(outputs[-1])
        except Exception as e:
            if "not loaded" in str(e).lower() or "not found" in str(e).lower():
                outputs.append("KelVPNCheck.service is not running or not found.")
                logger.warning(outputs[-1])
            else:
                outputs.append(f"KelVPNCheck.service stop failed: {e}")
                logger.error(outputs[-1])
        
        # Stop KelVPNService.service
        try:
            if self.is_service_active("KelVPNService.service"):
                cmd_stop_service: List[str] = ["systemctl", "stop", "KelVPNService.service"]
                service_output: str = self.execute_command(cmd_stop_service)
                outputs.append(f"KelVPNService.service stopped: {service_output}")
                logger.info(outputs[-1])
            else:
                outputs.append("KelVPNService.service is not running or not found.")
                logger.warning(outputs[-1])
        except Exception as e:
            if "not loaded" in str(e).lower() or "not found" in str(e).lower():
                outputs.append("KelVPNService.service is not running or not found.")
                logger.warning(outputs[-1])
            else:
                outputs.append(f"KelVPNService.service stop failed: {e}")
                logger.error(outputs[-1])
        
        # Update configuration
        try:
            self.update_config(is_connected=False)
        except Exception as e:
            logger.error(f"Error updating configuration: {e}")
        
        return "\n".join(outputs)


    def is_service_active(self, service_name: str) -> bool:
        """
        Checks whether the service exists and whether it is active.
        :param service_name: The name of the service.
        :return: True if the service is active, otherwise False.
        """
        try:
            cmd_check: List[str] = ["systemctl", "list-units", "--full", "--all", service_name]
            result: str = self.execute_command(cmd_check).strip()
        except Exception as e:
            logger.warning(f"Service check failed for {service_name}: {e}")
            return False
        
        if service_name in result:
            try:
                active_result = self.execute_command(["systemctl", "is-active", service_name]).strip()
                return active_result == "active"
            except Exception as e:
                logger.warning(f"Service is-active check failed for {service_name}: {e}")
                return False
        else:
            logger.warning(f"Service {service_name} is not found in systemd.")
            return False

    def restart_service(self) -> Tuple[str, bool]:
        """
        Restarts the KelVPNService and kelvpn_backend.service.

        :return: Tuple containing the output from the restart commands and success flag.
        """
        try:
            # Update configuration
            self.update_config(is_connected=False)

            # Restart KelVPNService
            cmd_restart_kelvpn: List[str] = [
                "systemctl",
                "restart",
                "KelVPNService.service",
            ]
            kelvpn_output: str = self.execute_command(cmd_restart_kelvpn)
            logger.info(f"KelVPNService restarted successfully: {kelvpn_output}")

            # Restart kelvpn_backend.service
            cmd_restart_backend: List[str] = [
                "systemctl",
                "restart",
                "kelvpn_backend.service",
            ]
            backend_output: str = self.execute_command(cmd_restart_backend)
            logger.info(
                f"kelvpn_backend.service restarted successfully: {backend_output}"
            )

            return kelvpn_output + "\n" + backend_output, True
        except Exception as e:
            logger.error(f"Error restarting services: {str(e)}")
            return str(e), False

    def reboot_system(self) -> str:
        """
        Reboots the system

        :return: Standard output from the reboot command.
        """
        cmd_reboot: List[str] = ["reboot"]
        output: str = self.execute_command(cmd_reboot)
        logger.info("System reboot initiated.")
        return output

    def get_service_status(self) -> str:
        """
        Retrieves the status of the KelVPNService.

        :return: Standard output from the status command.
        """
        cmd_status: List[str] = [
            "systemctl",
            "status",
            "KelVPNService",
            "--no-pager",
        ]
        output: str = self.execute_command(cmd_status)
        return output

    def set_serial(self, serial_key: str) -> Tuple[str, bool, str]:
        """
        Sets the serial key using the best server and updates the remaining days dynamically.

        :param serial_key: The serial key to activate.
        :return: Tuple containing the command output, success flag, and a message indicating the result.
        """
        servers_ping: List[Dict[str, Any]] = self.get_server_pings()

        best_server = min(servers_ping, key=lambda server: server["ping"])
        logger.info(f"best_server {best_server} for activation.")
        if best_server:
            server_id = best_server["server_id"]
            logger.info(f"Using best server with ID {server_id} for activation.")
        else:
            raise Exception("No server available for activation.")

        cmd_set_serial: List[str] = [
            self.cli_path,
            "-C",
            server_id,
            "-S",
            serial_key,
        ]
        logger.debug(f"command set serial: {cmd_set_serial}")
        output: str = self.execute_command(cmd_set_serial)
        logger.debug(f"Output: {output}")
        success, message = self._parse_set_serial_output(output)

        if not success:
            return output, success, message

        # Получаем оставшиеся дни из команды -AI
        remaining_days = self._get_serial_days_left()
        if remaining_days <= 0:
            logger.error("Failed to retrieve remaining days from serial activation.")
            return output, False, "Unable to activate serial key."

        # Создаем или обновляем существующую запись SerialHistory
        serial, created = SerialHistory.objects.update_or_create(
            serial_key=serial_key,
            defaults={
                "status": "active",
                "total_days": remaining_days,
                "days_active": 0,
            },
        )

        if created:
            logger.info(f"New serial key {serial.serial_key} created with {remaining_days} days.")
        else:
            serial.update_remaining_days()
            if serial.status != "active":
                serial.status = "active"
                serial.suspended_at = None
                serial.save()
                logger.info(f"Serial key {serial.serial_key} activated successfully.")
            if serial.status == "deactivated":
                serial.resume()
                logger.info(f"Serial key {serial.serial_key} resumed from suspension.")

        return output, success, message

    def _get_serial_days_left(self) -> int:
        """
        Retrieves the number of remaining days for the current serial key using KelVPNCli -AI.

        :return: Number of days left or -1 if parsing failed.
        """
        cmd_auth_info: List[str] = [self.cli_path, "-AI"]
        try:
            output = self.execute_command(cmd_auth_info)
            logger.debug(f"Authorization info output: {output}")

            match = re.search(r"days left: (\d+) days", output)
            if match:
                days_left = int(match.group(1))
                logger.info(f"Days left from serial key: {days_left}")
                return days_left

            logger.error("Failed to parse days left from authorization info.")
            return -1

        except Exception as e:
            logger.error(f"Error retrieving days left from serial key: {str(e)}")
            return -1

    def _parse_set_serial_output(self, output: str) -> Tuple[bool, str]:
        """
        Parses the output of the set_serial command to determine success and returns a message.

        :param output: The standard output from the set_serial command.
        :return: Tuple of boolean success flag and message.
        """
        logger.debug(f"Parsing command output: {output}")

        config: Optional[Config] = Config.objects.first()
        if not config:
            logger.error("Configuration not found.")
            raise Exception("Configuration not found.")

        # Check for a successful connection
        if "State: Connect" in output or "State: Connected" in output:
            config.is_activated = True
            config.is_connected = True
            config.save()
            logger.info("Serial key successfully activated and connected.")
            return True, "Serial key successfully activated and connection established."

        # Check for key activation without connection
        if "No activated key found" in output and "State: Connecting" in output:
            if "Error: code:" in output:
                error_code = re.search(r"Error: code: (\d+)", output)
                error_msg = f"Activation failed with error code {error_code.group(1) if error_code else 'unknown'}."
                logger.error(error_msg)
                return False, error_msg

        if "State: Disconnect" in output and "No activated key found" in output:
            logger.error("No activated key found. Connection attempt failed.")
            return False, "No activated key found. Connection attempt failed."

        if "Invalid serial key format" in output:
            logger.error(f"Invalid serial key format detected: {output}")
            return False, "Invalid serial key format."

        # Handle generic errors
        if "Error: code:" in output:
            error_code = re.search(r"Error: code: (\d+)", output)
            message = re.search(r"message: (.+)", output)
            error_msg = (
                f"Error code {error_code.group(1)}, message: {message.group(1)}"
                if error_code and message
                else "Unknown error occurred."
            )
            logger.error(f"Activation failed: {error_msg}")
            return False, error_msg

        # Unexpected output handling
        logger.error(f"Unexpected output: {output}")
        return False, "Unexpected error occurred. Check logs for details."

    def get_serial_history(self) -> List[Dict[str, Any]]:
        """
        Retrieves and stores the serial history.

        :return: List of dictionaries containing serial keys and their creation timestamps.
        """
        cmd_history: List[str] = [self.cli_path, "-H"]
        output: str = self.execute_command(cmd_history)
        serial_keys: List[str] = self._parse_serial_history(output)

        with transaction.atomic():
            for key in serial_keys:
                try:
                    obj, created = SerialHistory.objects.get_or_create(
                        serial_key=key, defaults={"status": "deactivated"}
                    )
                    if created:
                        logger.info(f"Created new SerialHistory for serial key {key}.")
                    else:
                        logger.warning(f"Serial key {key} already exists.")
                except Exception as e:
                    logger.error(
                        f"Error while creating or getting SerialHistory for serial key {key}. Error: {str(e)}"
                    )
                    pass

        history: List[SerialHistory] = SerialHistory.objects.all().order_by(
            "-created_at"
        )

        for entry in history:
            entry.update_remaining_days()
        logger.info(f"history: {list(history)}")
        return [
            {
                "serial_key": entry.serial_key,
                "created_at": entry.created_at,
                "status": entry.status,
                "total_days": entry.total_days,
                "days_active": entry.days_active,
                "remaining_days": entry.total_days - entry.days_active,
                "expired_date": entry.expired_date,
            }
            for entry in history
        ]

    def _parse_serial_history(self, output: str) -> List[str]:
        """
        Parses the output of the KelVPN -H command and returns a list of serial keys.

        :param output: The standard output from the KelVPNCli -H command.
        :return: List of serial keys.
        """
        serial_keys: List[str] = []
        lines: List[str] = output.splitlines()

        # Find the line with "Serial History size"
        size_line: Optional[str] = next(
            (line for line in lines if "Serial History size" in line), None
        )
        if size_line:
            try:
                size: int = int(
                    re.search(r"Serial History size: (\d+)", size_line).group(1)
                )
            except (AttributeError, ValueError):
                size = 0
        else:
            size = 0

        for line in lines:
            match = re.match(r"\d{2} : ([A-Z0-9-]{19})", line)
            if match:
                serial_keys.append(match.group(1))

        return serial_keys

    def deactivate_serial(self) -> Tuple[str, bool, Optional[SerialHistory]]:
        """
        Deactivates the serial key by resetting it using KelVPNCli and updates the serial history.

        :return: Tuple containing the command output, success flag, and the deactivated serial key object.
        """
        cmd_reset_serial: List[str] = [self.cli_path, "-W"]
        output: str = self.execute_command(cmd_reset_serial)
        logger.debug(f"Deactivation command output: {output}")

        # Проверка успешного выполнения команды
        if "Success!" in output and "Resetting the serial key." in output:
            active_serial = SerialHistory.objects.filter(status="active").first()

            if active_serial:
                # Обновление статуса серийного ключа
                active_serial.status = "deactivated"
                active_serial.suspended_at = timezone.now()
                active_serial.save()
                logger.info(
                    f"Serial key {active_serial.serial_key} deactivated successfully."
                )
            else:
                logger.warning("No active serial key found during deactivation.")

            # Обновление конфигурации
            self.update_config(is_connected=False, is_activated=False)

            return output, True, active_serial

        logger.error(f"Failed to deactivate serial key. Command output: {output}")
        return output, False, None

    def get_server_pings(self) -> List[Dict[str, Any]]:
        """
        Retrieves the list of servers with their pings and updates the database.

        :return: List of dictionaries containing server details.
        """
        cmd_servers: List[str] = [self.cli_path, "-L", "-R"]
        output: str = self.execute_command(cmd_servers)
        servers_ping: List[Dict[str, Any]] = self._parse_server_ping_list(output)

        with transaction.atomic():
            for server in servers_ping:
                Server.objects.update_or_create(
                    server_id=server["id"],
                    defaults={
                        "name": server["name"],
                        "ping": server["ping"],
                        "selected": False,
                    },
                )

        config, created = Config.objects.get_or_create(
            id=1,
            defaults={
                "ip": "0.0.0.0",
                "location": "",
                "dns": "8.8.8.8",
                "gateway": "192.168.100.1",
                "selected_server": None,
                "is_connected": False,
                "is_activated": False,
            },
        )

        logger.debug(f"Config: {config}")
        logger.debug(f"Selected server in config: {config.selected_server}")

        if config.selected_server:
            logger.debug(f"Using server from config: {config.selected_server}")

            Server.objects.filter(selected=True).update(selected=False)
            selected_server = Server.objects.filter(
                server_id=config.selected_server.server_id
            ).first()

            if selected_server:
                selected_server.selected = True
                selected_server.save()
            else:
                logger.debug(
                    f"Warning: Server from config not found in the current ping list."
                )
        else:
            logger.debug("No server is selected, selecting the best server...")

            Server.objects.filter(selected=True).update(selected=False)

            best_server: Optional[Server] = (
                Server.objects.filter(ping__gte=0).order_by("ping").first()
            )

            if best_server:
                config.selected_server = best_server
                best_server.selected = True
                best_server.save()
                config.save()
                logger.debug(
                    f"Automatically selected server: {best_server} (selected: {best_server.selected})"
                )
            else:
                logger.debug("No suitable server found to select.")

        return [
            {
                "server_id": server.server_id,
                "name": server.name,
                "ping": server.ping,
                "selected": server.selected,
            }
            for server in Server.objects.all().order_by("ping")
        ]

    def _parse_server_ping_list(self, output: str) -> List[Dict[str, Any]]:
        """
        Parses the output of the KelVPNCli -L -R command and returns a list of servers with pings.

        :param output: The standard output from the KelVPNCli -L -R command.
        :return: List of dictionaries containing server details.
        """
        servers_ping: List[Dict[str, Any]] = []
        lines: List[str] = output.splitlines()

        server_ping_line_regex = re.compile(
            r"^\s*(\d{2})\s*\|\s*(\d{1,4})\s*\|\s*(.+?)\s*$"
        )

        for line in lines:
            match = server_ping_line_regex.match(line)
            if match:
                server_id: str = match.group(1).strip()
                ping_str: str = match.group(2).strip()
                server_name: str = match.group(3).strip()

                try:
                    ping: int = int(ping_str)
                except ValueError:
                    ping = -1

                if server_id and server_name:
                    servers_ping.append(
                        {
                            "id": server_id,
                            "ping": ping,
                            "name": server_name,
                            "selected": False,
                        }
                    )
                    logger.debug(
                        f"Parsed server with ping: ID={server_id}, Ping={ping}, Name={server_name}"
                    )

        logger.info(f"Total servers with ping parsed: {len(servers_ping)}")
        return servers_ping

    def update_config(
        self, is_connected: bool, is_activated: Optional[bool] = None
    ) -> None:
        """
        Updates the config's connection and activation status.

        :param is_connected: Boolean indicating if the service is connected.
        :param is_activated: Optional boolean indicating if the service is activated.
        """
        config: Optional[Config] = Config.objects.first()
        if config:
            config.is_connected = is_connected
            if is_activated is not None:
                config.is_activated = is_activated
            config.save()

    def _get_ip_addresses(self):
        # result = subprocess.run(["nmcli", "device", "show"], capture_output=True, text=True)
        # addresses = re.findall(r"IP4\.ADDRESS\[\d+\]:\s+([\d.]+)", result.stdout)
        try:
            external_ip = (
                subprocess.check_output(["curl", "-s", "ifconfig.me"]).decode().strip()
            )
            # logger.debug(f"ip {external_ip}")
            return external_ip

        except subprocess.CalledProcessError as e:
            logger.debug(f"Error retrieving external IP: {e}")
            return None

    def _get_gateway(self):
        """
        Получает IP-адрес шлюза с приоритетом на tun0 (VPN), затем eth0 (Ethernet) и, наконец, wlan0 (Wi-Fi).
        """
        result = subprocess.run(
            ["nmcli", "device", "show"], capture_output=True, text=True
        )
        output = result.stdout

        # Определяем приоритет интерфейсов
        priority_interfaces = ["tun0", "eth0", "wlan0"]

        for interface in priority_interfaces:
            if f"GENERAL.DEVICE:                         {interface}" in output:
                match = re.search(
                    rf"{interface}\n(?:.*\n)*?IP4\.GATEWAY:\s+([\d.]+)", output
                )
                if match:
                    return match.group(1)

        return "-"

    def _get_dns_servers(self):
        dns_servers = []
        try:
            with open("/etc/resolv.conf", "r") as file:
                for line in file:
                    if line.startswith("nameserver"):
                        dns_servers.append(line.split()[1])
        except Exception as e:
            logger.error(f"Error retrieving DNS servers from resolv.conf: {str(e)}")
        return dns_servers

    def update_network_config(self):
        ip_addresses = self._get_ip_addresses()
        gateway = self._get_gateway()
        dns_servers = self._get_dns_servers()
        logger.info(f"ip {ip_addresses}, gateway {gateway}, dns_servers {dns_servers}")
        config, created = Config.objects.get_or_create(id=1)
        config.ip = ip_addresses if ip_addresses else "-"
        config.gateway = gateway
        config.dns = ", ".join(dns_servers)

        config.save()
        logger.info("Network configuration updated successfully.")


class WifiManager:
    CONFIG_PATHS = settings.HOSTAPD_CONFIG_PATHS

    @staticmethod
    def start_hostapd(interface: str) -> None:
        """
        Starts the hostapd service for the given interface.
        """
        subprocess.run(["systemctl", "start", f"hostapd@{interface}"], check=True)

    @staticmethod
    def stop_hostapd(interface: str) -> None:
        """
        Stops the hostapd service for the given interface.
        """
        subprocess.run(["systemctl", "stop", f"hostapd@{interface}"], check=True)

    @staticmethod
    def restart_hostapd(interface: str) -> None:
        """
        Restarts the hostapd service for the given interface.
        """
        subprocess.run(["systemctl", "restart", f"hostapd@{interface}"], check=True)

    @staticmethod
    def toggle_hostapd(interface: str, enable: bool) -> None:
        """
        Toggles the hostapd service based on the `enable` flag.
        """
        if enable:
            WifiManager.start_hostapd(interface)
        else:
            WifiManager.stop_hostapd(interface)

    @staticmethod
    def read_configs() -> list[dict]:
        """
        Reads configurations from all hostapd config files.
        Returns a list of configuration dictionaries.
        """
        configs = []
        for interface, config_path in WifiManager.CONFIG_PATHS.items():
            config = {"interface": interface, "config_file": config_path}
            try:
                with open(config_path, "r") as file:
                    for line in file:
                        line = line.strip()
                        if line and not line.startswith("#") and "=" in line:
                            key, value = line.split("=", 1)
                            config[key.strip()] = value.strip()
                configs.append(config)
            except FileNotFoundError:
                logger.error(f"Configuration file {config_path} not found.")
        return configs

    @staticmethod
    def determine_band(config: dict) -> str:
        """
        Determines the Wi-Fi band based on the configuration.
        """
        hw_mode = config.get("hw_mode", "g")
        channel = int(config.get("channel", 0))

        if hw_mode == "g" and 1 <= channel <= 14:
            return "2.4GHz"
        if hw_mode == "a" and 36 <= channel <= 165:
            return "5GHz"
        if hw_mode == "6" and 1 <= channel <= 233:
            return "6GHz"
        return "Unknown"

    @staticmethod
    def update_wifi_status() -> None:
        """
        Updates the WiFiStatus records based on configurations.
        """
        configs = WifiManager.read_configs()
        for config in configs:
            try:
                WiFiStatus.objects.update_or_create(
                    interface=config["interface"],
                    defaults={
                        "band": WifiManager.determine_band(config),
                        "ssid": config.get("ssid", "-"),
                        "channel": config.get("channel", "-"),
                        "hw_mode": config.get("hw_mode", "-"),
                        "transmit_power": config.get("transmit_power", 0),
                        "enabled": WifiManager.check_service_status(config["interface"]),
                        "wpa_passphrase": config.get("wpa_passphrase", "-"),
                    },
                )
            except Exception as e:
                logger.error(f"Failed to update Wi-Fi status: {e}")

    @staticmethod
    def check_service_status(interface: str) -> bool:
        """
        Checks whether the hostapd service for the given interface is active.
        """
        result = subprocess.run(
            ["systemctl", "is-active", f"hostapd@{interface}"],
            stdout=subprocess.PIPE,
            text=True,
        )
        return result.stdout.strip() == "active"

    @staticmethod
    def update_network_config(interface: str, **kwargs) -> None:
        """
        Updates Wi-Fi network configuration parameters in the hostapd config file.
        Only provided kwargs are updated.
        """
        config_path = WifiManager.CONFIG_PATHS.get(interface)
        if not config_path:
            raise FileNotFoundError(f"Configuration file for {interface} not found.")

        config_data = {}
        if os.path.exists(config_path):
            with open(config_path, "r") as file:
                for line in file:
                    line = line.strip()
                    if line and not line.startswith("#") and "=" in line:
                        key, value = line.split("=", 1)
                        config_data[key.strip()] = value.strip()

        for key, value in kwargs.items():
            if value is not None:
                config_data[key] = str(value)

        with open(config_path, "w") as file:
            for key, value in config_data.items():
                file.write(f"{key}={value}\n")

        WifiManager.restart_hostapd(interface)

class UpdateService:
    @staticmethod
    def check_updates():
        """
        Проверяет наличие обновлений.
        Возвращает True, если обновления доступны, False в противном случае.
        """
        try:
            result = subprocess.run(
                ["apt-get", "-s", "upgrade"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
            )

            if (
                "обновлено 0 пакетов, установлено 0 новых пакетов"
                in result.stdout.lower()
            ):
                return False
            elif "0 upgraded, 0 newly installed" in result.stdout:
                return False

            return True
        except Exception as e:
            logger.error(f"Error checking for updates: {str(e)}")
            raise

    @staticmethod
    def perform_update():
        update_status, created = UpdateStatus.objects.get_or_create(id=1)
        log_path = "/opt/kelvpn-web-admin/backend/logs/"
        if update_status.status == "IN_PROGRESS":
            return False, "Update already in progress"

        update_status.status = "IN_PROGRESS"
        update_status.log_file = (
            f"{log_path}update_log_{datetime.now().strftime('%Y%m%d_%H%M%S')}.txt"
        )
        update_status.save()

        def update_process():
            try:
                with open(update_status.log_file, "w") as f:
                    subprocess.run(
                        ["sudo", "apt-get", "update", "-y"],
                        stdout=f,
                        stderr=f,
                        text=True,
                    )

                    result_upgrade = subprocess.run(
                        ["sudo", "apt-get", "upgrade", "-y"],
                        stdout=f,
                        stderr=f,
                        text=True,
                    )

                    if result_upgrade.returncode != 0:
                        update_status.status = "FAILED"
                        f.write(f"Error during upgrade: {result_upgrade.stderr}\n")
                    else:
                        update_status.status = "COMPLETED"

                    update_status.completed_at = datetime.now()
                    update_status.save()

            except Exception as e:
                with open(update_status.log_file, "a") as f:
                    f.write(f"Error performing update: {str(e)}\n")
                update_status.status = "FAILED"
                update_status.completed_at = datetime.now()
                update_status.save()

        # Запуск задачи в фоновом режиме
        thread = threading.Thread(target=update_process)
        thread.start()

        return True, "Update started"

    @staticmethod
    def get_update_status():
        update_status, created = UpdateStatus.objects.get_or_create(id=1)
        return {
            "status": update_status.status,
            "log_file": f"logs/{update_status.log_file}",
            "started_at": update_status.started_at,
            "completed_at": update_status.completed_at,
        }


class NetworkManager:
    WAN_INTERFACE = "wan0"

    @staticmethod
    def get_network_settings():
        settings = {
            "mac_address": NetworkManager.get_mac_address(),
            "ip_address": NetworkManager.get_ip_address(),
            "gateway": NetworkManager.get_default_gateway(),
            "ip_mode": NetworkManager.get_ip_mode(),
        }
        return settings

    @staticmethod
    def get_mac_address():
        result = subprocess.run(
            ["cat", f"/sys/class/net/{NetworkManager.WAN_INTERFACE}/address"],
            stdout=subprocess.PIPE,
            text=True,
        )
        return result.stdout.strip()

    @staticmethod
    def set_mac_address(mac_address: str):
        if not re.match(r"^([0-9A-Fa-f]{2}:){5}[0-9A-Fa-f]{2}$", mac_address):
            raise ValueError("Invalid MAC address format.")
        
        subprocess.run(
            ["ip", "link", "set", NetworkManager.WAN_INTERFACE, "down"]
        )
        subprocess.run(
            ["ip", "link", "set", NetworkManager.WAN_INTERFACE, "address", mac_address]
        )
        subprocess.run(
            ["ip", "link", "set", NetworkManager.WAN_INTERFACE, "up"]
        )

    @staticmethod
    def get_ip_address():
        result = subprocess.run(
            ["ip", "-4", "addr", "show", NetworkManager.WAN_INTERFACE],
            stdout=subprocess.PIPE,
            text=True,
        )
        match = re.search(r"inet (\d+\.\d+\.\d+\.\d+)", result.stdout)
        return match.group(1) if match else "Not Assigned"

    @staticmethod
    def get_default_gateway():
        result = subprocess.run(
            ["ip", "route", "show", "default"],
            stdout=subprocess.PIPE,
            text=True,
        )
        match = re.search(r"default via (\d+\.\d+\.\d+\.\d+)", result.stdout)
        return match.group(1) if match else "Not Assigned"

    @staticmethod
    def get_ip_mode():
        dhcp_file = f"/var/lib/dhcp/dhclient.{NetworkManager.WAN_INTERFACE}.leases"
        if os.path.exists(dhcp_file):
            return "Dynamic"
        return "Static"

    @staticmethod
    def set_ip_config(static: bool, ip_address=None, netmask=None, gateway=None):
        if static:
            if not all([ip_address, netmask, gateway]):
                raise ValueError("IP address, netmask, and gateway must be set.")
            subprocess.run(
                [
                    "ip", "addr", "flush", "dev", NetworkManager.WAN_INTERFACE
                ]
            )
            subprocess.run(
                [
                    "ip", "addr", "add", f"{ip_address}/{netmask}",
                    "dev", NetworkManager.WAN_INTERFACE,
                ]
            )
            subprocess.run(
                ["ip", "route", "add", "default", "via", gateway]
            )
        else:
            subprocess.run(["dhclient", NetworkManager.WAN_INTERFACE])